@@ -2,7 +2,7 @@
 // random_access_file.cpp
 // ~~~~~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2022 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2025 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -31,22 +31,18 @@ struct write_some_at_handler
 {
   write_some_at_handler() {}
   void operator()(const asio::error_code&, std::size_t) {}
-#if defined(ASIO_HAS_MOVE)
   write_some_at_handler(write_some_at_handler&&) {}
 private:
   write_some_at_handler(const write_some_at_handler&);
-#endif // defined(ASIO_HAS_MOVE)
 };
 
 struct read_some_at_handler
 {
   read_some_at_handler() {}
   void operator()(const asio::error_code&, std::size_t) {}
-#if defined(ASIO_HAS_MOVE)
   read_some_at_handler(read_some_at_handler&&) {}
 private:
   read_some_at_handler(const read_some_at_handler&);
-#endif // defined(ASIO_HAS_MOVE)
 };
 
 void test()
@@ -78,22 +74,18 @@ void test()
     random_access_file::native_handle_type native_file2 = file1.native_handle();
     random_access_file file8(ioc_ex, native_file2);
 
-#if defined(ASIO_HAS_MOVE)
     random_access_file file9(std::move(file8));
 
     basic_random_access_file<io_context::executor_type> file10(ioc);
     random_access_file file11(std::move(file10));
-#endif // defined(ASIO_HAS_MOVE)
 
     // basic_random_access_file operators.
 
-#if defined(ASIO_HAS_MOVE)
     file1 = random_access_file(ioc);
     file1 = std::move(file2);
     file1 = std::move(file10);
-#endif // defined(ASIO_HAS_MOVE)
 
-    // basic_io_object functions.
+    // I/O object functions.
 
     random_access_file::executor_type ex = file1.get_executor();
     (void)ex;
